<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 *
 * With the purchase or the installation of the software in your application
 *
 * you accept the licence agreement.
 *
 *
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Banca Sella S.p.A
 *  @copyright 1996-2024 Banca Sella S.p.A
 *  @license   LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class APPpago extends PaymentModule
{
    public function __construct()
    {
        // set name
        $this->name = 'apppago';
        // set tab
        $this->tab = 'payments_gateways';
        // set version
        $this->version = '1.3.0';
        // set author
        $this->author = 'Banca Sella';
        // set need_instance
        $this->need_instance = 0;
        // set dependencies
        $this->dependencies = [];
        // set ps_versions_compliancy
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
        // set bootstrap
        $this->bootstrap = true;
        // call parent constructor
        parent::__construct();
        // set displayName
        $this->displayName = 'AppPago';
        // set description
        $this->description = 'AppPago Payment Gateway';
        // set confirmUninstall
        $this->confirmUninstall = 'Are you sure you want to uninstall?';
        // enable configuration
        $this->controllers = ['config'];
    }

    public function install()
    {
        if (!parent::install() or !$this->registerHook('paymentOptions') or !$this->registerHook('paymentReturn') or !$this->registerHook('displayProductAdditionalInfo')) {
            return false;
        }

        return true;
    }

    // uninstall() method
    public function uninstall()
    {
        // call parent uninstall method
        if (!parent::uninstall()) {
            return false;
        }

        return true;
    }

    public function postProcess()
    {
        // names are the same as the ones in config.xml
        $merchantCode = Tools::getValue('merchantCode');
        $testingMode = Tools::getValue('testingMode');
        $vat = Tools::getValue('vat');
        $api_key = bin2hex(random_bytes(16));
        $min_amount = Tools::getValue('min_amount');
        $max_amount = Tools::getValue('max_amount');
        $installments_type = Tools::getValue('installments_type');
        // cast to string iban_allowed and company_allowed
        $iban_allowed = (string) Tools::getValue('iban_allowed') ? '1' : '0';
        $company_allowed = (string) Tools::getValue('company_allowed') ? '1' : '0';
        $installments_type = Tools::getValue('installments_type');

        // Campi dinamici
        $fixed_installments_number = '';
        $range_installments_min = '';
        $range_installments_max = '';
        $select_installments_list = [];

        if ($installments_type === 'fixed') {
            $fixed_installments_number = Tools::getValue('fixed_installments_number');
        } elseif ($installments_type === 'range') {
            $range_installments_min = Tools::getValue('range_installments_min');
            $range_installments_max = Tools::getValue('range_installments_max');
        } elseif ($installments_type === 'select') {
            $select_installments_list = Tools::getValue('select_installments_list');
        }

        $base_url = 'https://secure.sellapersonalcredit.it/SPCPlugAppPago/api/config';

        $postFields =
            [
                'platform' => 'prestashop',
                'version' => $this->version,
                'host' => $_SERVER['HTTP_HOST'],
                'merchant_code' => $merchantCode,
                'vat' => $vat,
                'api_key' => $api_key,
                'max_amount' => $max_amount,
                'min_amount' => $min_amount,
                'installments_type' => $installments_type,
                'fixed_installments_number' => $fixed_installments_number,
                'range_installments_min' => $range_installments_min,
                'range_installments_max' => $range_installments_max,
                'select_installments_list' => $select_installments_list,
                'iban_allowed' => $iban_allowed,
                'company_allowed' => $company_allowed,
            ];

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $base_url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postFields));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec($ch);

        curl_close($ch);

        $decodedResponse = json_decode($response, true);

        if (isset($decodedResponse['status']) && $decodedResponse['status'] === 'ok') {
            // names correspond to the ones in database table APPpago
            Configuration::updateValue('APPPAGO_TESTING_MODE', $testingMode);
            Configuration::updateValue('APPPAGO_MERCHANT_CODE', $merchantCode);
            Configuration::updateValue('APPPAGO_VAT', $vat);
            Configuration::updateValue('APPPAGO_API_KEY', $api_key);
            Configuration::updateValue('APPPAGO_MIN_INSTALLMENTS', $min_amount);
            Configuration::updateValue('APPPAGO_MAX_INSTALLMENTS', $max_amount);
            Configuration::updateValue('APPPAGO_INSTALLMENTS_TYPE', $installments_type);
            Configuration::updateValue('APPPAGO_FIXED_INSTALLMENTS_NUMBER', $fixed_installments_number);
            Configuration::updateValue('APPPAGO_RANGE_INSTALLMENTS_MIN', $range_installments_min);
            Configuration::updateValue('APPPAGO_RANGE_INSTALLMENTS_MAX', $range_installments_max);
            Configuration::updateValue('APPPAGO_SELECT_INSTALLMENTS_LIST', $select_installments_list);
            Configuration::updateValue('APPPAGO_IBAN_ALLOWED', $iban_allowed);
            Configuration::updateValue('APPPAGO_COMPANY_ALLOWED', $company_allowed);
        } else {
            $this->_errors[] = 'Errore durante la configurazione del modulo [ERRROR_CODE: 500]. Riprova più tardi. (La configurazione non è stata salvata)';
        }
        $this->_confirmations[] = 'Configurazione salvata con successo';
    }

    public function getContent()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            $this->postProcess();
        }

        if (count($this->_confirmations) > 0) {
            // Assegna le conferme a Smarty per la visualizzazione
            $this->context->smarty->assign('confirmations', $this->_confirmations);
        }

        if (count($this->_errors) > 0) {
            // Assegna gli errori a Smarty per la visualizzazione
            $this->context->smarty->assign('errors', $this->_errors);
        }

        $configArray = [
            'testingMode' => Configuration::get('APPPAGO_TESTING_MODE'),
            'merchantCode' => Configuration::get('APPPAGO_MERCHANT_CODE'),
            'vat' => Configuration::get('APPPAGO_VAT'),
            'api_key' => Configuration::get('APPPAGO_API_KEY'),
            'min_amount' => Configuration::get('APPPAGO_MIN_INSTALLMENTS'),
            'max_amount' => Configuration::get('APPPAGO_MAX_INSTALLMENTS'),
            'installments_type' => Configuration::get('APPPAGO_INSTALLMENTS_TYPE'),
            'fixed_installments_number' => Configuration::get('APPPAGO_FIXED_INSTALLMENTS_NUMBER'),
            'range_installments_min' => Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MIN'),
            'range_installments_max' => Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MAX'),
            'select_installments_list' => Configuration::get('APPPAGO_SELECT_INSTALLMENTS_LIST'),
            'iban_allowed' => Configuration::get('APPPAGO_IBAN_ALLOWED'),
            'company_allowed' => Configuration::get('APPPAGO_COMPANY_ALLOWED'),
        ];

        $this->context->smarty->assign($configArray);

        return $this->display(__FILE__, 'views/templates/admin/config.tpl');
    }

    public function getPaymentOptions()
    {
        if (!$this->active) {
            return;
        }

        // URL per l'azione del form
        $formAction = $this->context->link->getModuleLink($this->name, 'validation', [], true);

        // Assegna l'URL all'azione alla variabile del template
        $this->smarty->assign(['action' => $formAction]);

        // Carica il template del form
        $paymentForm = $this->fetch('module:apppago/views/templates/front/payment_options_priv.tpl');

        $logo_path = _PS_MODULE_DIR_ . $this->name . '/Apppago_small.webp';
        // Crea un nuovo oggetto PaymentOption
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $newOption->setModuleName($this->displayName)
            ->setCallToActionText('Paga in comode quote con ')
            ->setAction($formAction)
            ->setForm($paymentForm);
        $newOption->setLogo(Media::getMediaPath($logo_path));

        return [$newOption];
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        return $this->getPaymentOptions($params);
    }

    public function hookDisplayProductAdditionalInfo($params)
    {
        $product = $params['product'];
        $priceInt = $product['price'];
        $toPrint = 0;
        $priceInt = (int) substr($priceInt, 0, 3);
        if ($priceInt >= 200) {
            $price = $product['price'];
            $price = (float) substr($price, 0, 6);

            $installments = $price / 30;
            $installments = floor($installments);
            if ($installments > 12) {
                $installments = 12;
            }

            $toPrint = ceil(($price / $installments) * 100) / 100;
            if (strpos($toPrint, '.') === false) {
                $toPrint = ceil(($price / $installments) * 100) / 100 . '.00';
            }
            return $this->fetch('module:apppago/views/templates/front/widget.tpl', ['installments' => $installments, 'toPrint' => $toPrint, 'imgsrc' => _PS_MODULE_DIR_ . $this->name . '/apppago.webp']);
        }
    }
}
