<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 *
 * With the purchase or the installation of the software in your application
 *
 * you accept the licence agreement.
 *
 *
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Banca Sella S.p.A
 *  @copyright 1996-2024 Banca Sella S.p.A
 *  @license   LICENSE.txt
 */
use GuzzleHttp\Client;

if (!defined('_PS_VERSION_')) {
    exit;
}

class APPpagoOrderConfirmationModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();

        $result = Tools::getValue('result');
        $merchantTx = Tools::getValue('merchantTx');
        $contract_number = Tools::getValue('contract_number');
        $checkDigit = Tools::getValue('checkDigit');
        $order = new Order((int) $checkDigit);
        $customer = new Customer((int) $order->id_customer);

        $awaiting_payment = 'awaiting-payment';

        $api_url = 'https://sandbox.sellapersonalcredit.it/SPCPlugAppPago/api/order/update/status';

        if (Validate::isLoadedObject($order)) {
            $customer = new Customer((int) $order->id_customer);

            if (Validate::isLoadedObject($customer)) {
                $this->context->smarty->assign(
                    [
                        'order' => $order,
                        'firstname' => $customer->firstname,
                        'lastname' => $customer->lastname,
                        'result' => $result,
                        'merchantTx' => $merchantTx,
                        'contract_number' => $contract_number,
                        'checkDigit' => $checkDigit,
                    ]
                );

                // new client guzzle
                $client = new Client();

                // Verifica che l'ordine sia un oggetto valido
                if ($order && Validate::isLoadedObject($order)) {
                    if ($result == 'OK') {
                        $order->setCurrentState(Configuration::get('PS_OS_PREPARATION'));
                        $order->save();
                        $body = [
                            'body' => [
                                'merchant_order_id' => $checkDigit,
                                'olMerchantTrxId' => $merchantTx,
                                'new_status' => $awaiting_payment,
                                'contract_number' => $contract_number,
                            ],
                        ];

                        $response = $client->post($api_url, $body);

                        $statusCode = $response->getStatusCode();
                        $data = json_decode($response->getBody(), true);
                    }

                    if ($result == 'ERROR') {
                        $order->setCurrentState(Configuration::get('PS_OS_ERROR'));
                        $order->save();

                        $body = [
                            'body' => [
                                'merchant_order_id' => $checkDigit,
                                'olMerchantTrxId' => $merchantTx,
                                'new_status' => 'failed',
                                'contract_number' => $contract_number,
                            ],
                        ];

                        $response = $client->post($api_url, $body);

                        $statusCode = $response->getStatusCode();
                        $data = json_decode($response->getBody(), true);
                    }

                    if (!$customer || !Validate::isLoadedObject($customer)) {
                        exit('Errore: il cliente non è valido.');
                    }
                } else {
                    exit("Errore: l'ordine non è valido.");
                }

                $this->setTemplate('module:apppago/views/templates/front/custom_order_confirmation.tpl');
            } else {
                Tools::redirect('index.php?controller=404');
            }
        } else {
            Tools::redirect('index.php?controller=404');
        }
    }
}
