<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 *
 * With the purchase or the installation of the software in your application
 *
 * you accept the licence agreement.
 *
 *
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Banca Sella S.p.A
 *  @copyright 1996-2024 Banca Sella S.p.A
 *  @license   LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
use Symfony\Component\HttpFoundation\JsonResponse;

class APPpagoSetStateOrderModuleFrontController extends ModuleFrontController
{
    public function run()
    {
        $this->init();
        $this->postProcess();
    }

    public function postProcess()
    {
        $apiKey = Configuration::get('APPPAGO_API_KEY');
        $headers = getallheaders();

        if (!isset($headers['X-Api-Key']) || $headers['X-Api-Key'] != $apiKey) {
            // set status code to 401
            $response = new JsonResponse(['status_code' => $headers, 'status' => 'error', 'message' => 'Unauthorized']);
            $response->setStatusCode(401);
            $response->send();
            exit;
        }

        $merchant_order_id = Tools::getValue('merchant_order_id');
        $new_status = Tools::getValue('new_status');
        $order = new Order((int) $merchant_order_id);

        if (Validate::isLoadedObject($order)) {
            if ($new_status === 'completed') {
                $order->setCurrentState(Configuration::get('PS_OS_PREPARATION'));
            } elseif ($new_status === 'refused') {
                $order->setCurrentState(Configuration::get('PS_OS_ERROR'));
            }
            $response = new JsonResponse(['status' => 'success', 'message' => 'Order updated']);
            $response->setStatusCode(200);
        } else {
            $response = new JsonResponse(['status' => 'error', 'message' => 'Order not found']);
            $response->setStatusCode(404);
        }
        $response->send();
        exit;
    }
}
