<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 *
 * With the purchase or the installation of the software in your application
 *
 * you accept the licence agreement.
 *
 *
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Banca Sella S.p.A
 *  @copyright 1996-2024 Banca Sella S.p.A
 *  @license   LICENSE.txt
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use GuzzleHttp\Client;

class APPpagoValidationModuleFrontController extends ModuleFrontController
{
    private $paymentModule;

    public function __construct($paymentModule)
    {
        $this->paymentModule = $paymentModule;
        parent::__construct();
    }

    public function initContent()
    {
        parent::initContent();
        $this->setTemplate('module:apppago/views/templates/front/payment_options_priv.tpl');
    }

    public function postProcess()
    {
        if (!($this->paymentModule instanceof PaymentModule)) {
            $this->paymentModule = Module::getInstanceByName('apppago');
            if (!($this->paymentModule instanceof PaymentModule)) {
                PrestaShopLogger::addLog('Failed to re-instantiate PaymentModule.', 3);
            }
        }
        // get order id and order details
        $cart = $this->context->cart;

        // Get customer
        $customer = new Customer((int) $cart->id_customer);

        // Get address
        $address = new Address((int) $cart->id_address_invoice);

        $callback_url = $this->context->link->getModuleLink($this->module->name, 'OrderConfirmation', [], true);

        $order_id = $cart->id;

        $olMerchantTrxId = substr(str_shuffle('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'), 0, 5);

        $body = [
            'body' => [
                'olMerchantTrxId' => $olMerchantTrxId,
                'merchantId' => Configuration::get('APPPAGO_MERCHANT_CODE'),
                'loanTotalAmount' => '' . $cart->getOrderTotal(true, Cart::BOTH) . '',
                'installmentsNumber' => Configuration::get('APPPAGO_FIXED_INSTALLMENTS_NUMBER'),
                'taxCode' => strtoupper(Tools::getValue('cf')) ?? '',
                'merchantRedirect' => $callback_url,
                'firstname' => $customer->firstname,
                'lastname' => $customer->lastname,
                'residenceAddress' => $address->address1 . ' ' . $address->address2,
                'residenceCity' => $address->city,
                'residenceState' => State::getNameById($address->id_state),
                'residencePostalCode' => $address->postcode,
                'email' => $customer->email,
                'phone' => $address->phone ?? $address->phone_mobile ?? '',
                'merchant_order_id' => '' . $order_id . '',
            ],
        ];

        $apiUrl = 'https://secure.sellapersonalcredit.it/SPCPlugAppPago/api/order';

        if (Configuration::get('APPPAGO_TESTING_MODE') == 'enabled') {
            $apiUrl = 'https://sandbox.sellapersonalcredit.it/SPCPlugAppPago/api/order';
        }

        $client = new Client();
        $response = $client->post($apiUrl, $body);

        $statusCode = $response->getStatusCode();
        $data = json_decode($response->getBody(), true);

        if (isset($data['redirectUrl'])) {
            // create order
            $this->paymentModule->validateOrder(
                // fake error here, see first if statement above
                $cart->id,
                Configuration::get('PS_OS_CHEQUE'),
                $cart->getOrderTotal(true, Cart::BOTH),
                $this->module->displayName,
                null,
                [],
                null,
                false,
                $customer->secure_key
            );

            Tools::redirect($data['redirectUrl']);
        }

        if (isset($data['status']) && $data['status'] == 'error') {
            exit('ERROR: ' . $data['error'] . $statusCode);
        }

        if (!isset($data['redirectUrl'])) {
            echo 'ERROR: Missing redirect URL' . $statusCode;
        }
    }
}
